use eframe::egui;

use crate::{main_state::MainState, transfer_menu::{CurrentTransferState, CurrentTransferStateInterior}};

use std::sync::LazyLock;

static SELECT_COLOR: LazyLock<egui::Color32> =
    LazyLock::new(|| egui::Color32::from_hex("#aa0000").unwrap());

pub fn tree(ui: &mut egui::Ui, ms: &mut MainState, cts: &CurrentTransferState) {
    // Add all source plates
    ui.vertical(|ui| {
        ui.heading("Source Plates");
        let mut new_uuid: Option<plate_tool_lib::uuid::Uuid> = None;
        for (name, uuid) in ms.source_plates.iter().map(|x| (&x.name, x.get_uuid())) {
            let f = {
                let mut f = egui::Frame::none();
                if ms.get_current_source().is_some_and(|x| x == uuid) {
                    f = f.fill(*SELECT_COLOR);
                }
                f
            };
            f.show(ui, |ui| {
                let r = ui.add(
                    egui::Label::new(name)
                        .sense(egui::Sense::click())
                        .selectable(false),
                );
                if r.clicked() {
                    new_uuid = Some(uuid);
                    log::info!("{:?}", uuid);
                }
            });
        }
        if let Some(uuid) = new_uuid {
            ms.set_current_source(uuid);
        }
    });

    // Add all destination plates
    ui.vertical(|ui| {
        ui.heading("Destination Plates");
        let mut new_uuid: Option<plate_tool_lib::uuid::Uuid> = None;
        for (name, uuid) in ms
            .destination_plates
            .iter()
            .map(|x| (&x.name, x.get_uuid()))
        {
            let f = {
                let mut f = egui::Frame::none();
                if ms.get_current_destination().is_some_and(|x| x == uuid) {
                    f = f.fill(*SELECT_COLOR);
                }
                f
            };
            f.show(ui, |ui| {
                let r = ui.add(
                    egui::Label::new(name)
                        .sense(egui::Sense::click())
                        .selectable(false),
                );
                if r.clicked() {
                    new_uuid = Some(uuid);
                    log::info!("{:?}", uuid);
                }
            });
        }
        if let Some(uuid) = new_uuid {
            ms.set_current_destination(uuid);
        }
    });
    // Add all transfers
    ui.vertical(|ui| {
        ui.heading("Transfers");
        let mut new_uuid: Option<plate_tool_lib::uuid::Uuid> = None;
        for (name, uuid) in ms.transfers.iter().map(|x| (&x.name, x.get_uuid())) {
            let f = {
                let mut f = egui::Frame::none();
                if ms.get_current_transfer().is_some_and(|x| x == uuid) {
                    f = f.fill(*SELECT_COLOR);
                }
                f
            };
            f.show(ui, |ui| {
                let r = ui.add(
                    egui::Label::new(name)
                        .sense(egui::Sense::click())
                        .selectable(false),
                );
                if r.clicked() {
                    new_uuid = Some(uuid);
                    log::info!("{:?}", uuid);
                }
            });
        }
        if let Some(uuid) = new_uuid {
            ms.set_current_transfer(uuid);
            if let Some(new_cts) = CurrentTransferStateInterior::try_new_from_main_state_transfer(ms) {
                *cts.lock().unwrap() = new_cts;
            }
        }
    });
}
